#include "ukmedia_app_item_widget.h"

UkmediaAppItemWidget::UkmediaAppItemWidget(QWidget *parent)
    : QWidget(parent)
{
    initUI();
}

void UkmediaAppItemWidget::initUI()
{
    this->setFixedSize(560,477);
    m_pAppLabel = new QLabel(tr("Application"), this);
    m_pAppLabel->setContentsMargins(17,0,0,0);

    m_pAppWidget = new QFrame(this);
    m_pInputWidget = new QFrame(m_pAppWidget);
    m_pOutputWidget = new QFrame(m_pAppWidget);
    m_pVolumeWidget = new QFrame(m_pAppWidget);
    m_pCheckWidget = new QFrame(m_pAppWidget);

    m_pAppWidget->setFrameShape(QFrame::Shape::Box);
    m_pInputWidget->setFrameShape(QFrame::Shape::Box);
    m_pOutputWidget->setFrameShape(QFrame::Shape::Box);
    m_pVolumeWidget->setFrameShape(QFrame::Shape::Box);
    m_pCheckWidget->setFrameShape(QFrame::Shape::Box);

    m_pInputWidget->setObjectName("InputWidget");
    m_pOutputWidget->setObjectName("OutputWidget");
    m_pVolumeWidget->setObjectName("VolumeWidget");

    m_pInputWidget->setStyleSheet("#InputWidget{border-radius: 6px; background-color: #F5F5F5;}");
    m_pOutputWidget->setStyleSheet("#OutputWidget{border-radius: 6px; background-color: #F5F5F5;}");
    m_pVolumeWidget->setStyleSheet("#VolumeWidget{border-radius: 6px; background-color: #F5F5F5;}");

    m_pAppWidget->setFixedSize(512, 415);
    m_pInputWidget->setFixedSize(512, 60);
    m_pOutputWidget->setFixedSize(512, 60);
    m_pVolumeWidget->setFixedSize(512, 60);
    m_pCheckWidget->setFixedSize(512, 36);

    //应用音量布局
    m_pVolumeLabel = new QLabel(tr("Output Volume"), m_pVolumeWidget);
    m_pVolumeBtn = new UkuiButtonDrawSvg(m_pVolumeWidget);
    m_pVolumeSlider = new KSlider(m_pVolumeWidget);
    m_pVolumeSlider->setRange(0,100);
    m_pVolumeSlider->setSliderType(SmoothSlider);
    m_pVolumeSlider->setNodeVisible(false);
    m_pVolumeNumLabel = new QLabel("0%", m_pVolumeWidget);

    m_pVolumeBtn->setFixedSize(24,24);
    m_pVolumeSlider->setFixedSize(260,55);
    m_pVolumeLabel->setFixedSize(80,20);
    m_pVolumeNumLabel->setFixedSize(55,20);
    m_pVolumeSlider->setOrientation(Qt::Horizontal);
    m_pVolumeNumLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);

    QHBoxLayout *volumeLayout = new QHBoxLayout(m_pVolumeWidget);
    volumeLayout->addItem(new QSpacerItem(17,20,QSizePolicy::Fixed));
    volumeLayout->addWidget(m_pVolumeLabel);
    volumeLayout->addItem(new QSpacerItem(50,20,QSizePolicy::Fixed));
    volumeLayout->addWidget(m_pVolumeBtn);
    volumeLayout->addItem(new QSpacerItem(10,20,QSizePolicy::Fixed));
    volumeLayout->addWidget(m_pVolumeSlider);
//    volumeLayout->addItem(new QSpacerItem(14,20,QSizePolicy::Fixed));
    volumeLayout->addWidget(m_pVolumeNumLabel);
    volumeLayout->addItem(new QSpacerItem(16,20,QSizePolicy::Fixed));
    volumeLayout->setSpacing(0);
    m_pVolumeWidget->setLayout(volumeLayout);
    m_pVolumeWidget->layout()->setContentsMargins(0,0,0,0);

    //输入设备布局
    m_pInputDevLabel = new QLabel(tr("Input Device"), m_pInputWidget);
    m_pInputCombobox = new QComboBox(m_pInputWidget);
    m_pInputDevLabel->setFixedSize(80,20);
    m_pInputCombobox->setFixedSize(347, 36);

    QHBoxLayout *inputLayout = new QHBoxLayout(m_pInputWidget);
    inputLayout->addItem(new QSpacerItem(17,20,QSizePolicy::Fixed));
    inputLayout->addWidget(m_pInputDevLabel);
    inputLayout->addItem(new QSpacerItem(52,20,QSizePolicy::Expanding));
    inputLayout->addWidget(m_pInputCombobox);
    inputLayout->addItem(new QSpacerItem(16,20,QSizePolicy::Fixed));
    inputLayout->setSpacing(0);
    m_pInputWidget->setLayout(inputLayout);
    m_pInputWidget->layout()->setContentsMargins(0,0,0,0);

    //输入设备布局
    m_pOutputDevLabel = new QLabel(tr("Output Device"), m_pOutputWidget);
    m_pOutputCombobox = new QComboBox(m_pOutputWidget);
    m_pOutputDevLabel->setFixedSize(80,20);
    m_pOutputCombobox->setFixedSize(347, 36);

    QHBoxLayout *outputLayout = new QHBoxLayout(m_pOutputWidget);
    outputLayout->addItem(new QSpacerItem(17,20,QSizePolicy::Fixed));
    outputLayout->addWidget(m_pOutputDevLabel);
    outputLayout->addItem(new QSpacerItem(52,20,QSizePolicy::Expanding));
    outputLayout->addWidget(m_pOutputCombobox);
    outputLayout->addItem(new QSpacerItem(16,20,QSizePolicy::Fixed));
    outputLayout->setSpacing(0);
    m_pOutputWidget->setLayout(outputLayout);
    m_pOutputWidget->layout()->setContentsMargins(0,0,0,0);

    // check Btn
    m_pCheckBtn = new QPushButton(tr("Check"), m_pCheckWidget);
    m_pCheckBtn->setFixedSize(96, 36);

    QHBoxLayout *checkLayout = new QHBoxLayout(m_pCheckWidget);
    checkLayout->addStretch();
    checkLayout->addWidget(m_pCheckBtn);
    checkLayout->setSpacing(0);
    m_pCheckWidget->setLayout(checkLayout);
    m_pCheckWidget->layout()->setContentsMargins(0,0,0,0);

//    line1 = myLine();
//    line2 = myLine();

    QVBoxLayout *vlayout = new QVBoxLayout(m_pAppWidget);
    vlayout->addWidget(m_pVolumeWidget);
    vlayout->addSpacing(1);
    vlayout->addWidget(m_pOutputWidget);
    vlayout->addSpacing(1);
    vlayout->addWidget(m_pInputWidget);
    vlayout->addItem(new QSpacerItem(16,197,QSizePolicy::Fixed));
    vlayout->addWidget(m_pCheckWidget);
    vlayout->setSpacing(0);
    vlayout->setContentsMargins(0,0,0,0);

    //整体布局
    QVBoxLayout *appLayout = new QVBoxLayout(this);
    appLayout->addItem(new QSpacerItem(16,8,QSizePolicy::Fixed));
    appLayout->addWidget(m_pAppLabel);
    appLayout->addItem(new QSpacerItem(16,8,QSizePolicy::Fixed));
    appLayout->addWidget(m_pAppWidget);
    appLayout->addItem(new QSpacerItem(16,24,QSizePolicy::Fixed));
    appLayout->addStretch();

    appLayout->setSpacing(0);
    this->setLayout(appLayout);
    this->layout()->setContentsMargins(25,0,23,0);
}

//  设置窗口应用标题
void UkmediaAppItemWidget::setTitleName(QString name)
{
    m_pAppLabel->setText(name);
}

void UkmediaAppItemWidget::setChildObjectName(QString name)
{
    m_pVolumeNumLabel->setObjectName(name+"-label");
    m_pVolumeBtn->setObjectName(name);
    m_pVolumeSlider->setObjectName(name);
    m_pInputCombobox->setObjectName(name+"-input");
    m_pOutputCombobox->setObjectName(name+"-output");
    this->setObjectName(name);
}

void UkmediaAppItemWidget::setSliderValue(int value)
{
    m_pVolumeSlider->blockSignals(true);
    m_pVolumeSlider->setValue(value);
    m_pVolumeSlider->blockSignals(false);

    QString percent = QString::number(value);
    m_pVolumeNumLabel->setText(percent+"%");
}

void UkmediaAppItemWidget::addInputCombobox(QString port)
{
    m_pInputCombobox->blockSignals(true);
    m_pInputCombobox->addItem(port);
    m_pInputCombobox->blockSignals(false);
}

void UkmediaAppItemWidget::addOutputCombobox(QString port)
{
    m_pOutputCombobox->blockSignals(true);
    m_pOutputCombobox->addItem(port);
    m_pOutputCombobox->blockSignals(false);
}

//  更新音量图标
void UkmediaAppItemWidget::outputVolumeDarkThemeImage(int value, bool status)
{
    QImage image;
    QColor color = QColor(0,0,0,216);

    if (status) {
        image = QIcon::fromTheme("audio-volume-muted-symbolic").pixmap(24,24).toImage();
    }
    else if (value <= 0) {
        image = QIcon::fromTheme("audio-volume-muted-symbolic").pixmap(24,24).toImage();
    }
    else if (value > 0 && value <= 33) {
        image = QIcon::fromTheme("audio-volume-low-symbolic").pixmap(24,24).toImage();
    }
    else if (value >33 && value <= 66) {
        image = QIcon::fromTheme("audio-volume-medium-symbolic").pixmap(24,24).toImage();
    }
    else {
        image = QIcon::fromTheme("audio-volume-high-symbolic").pixmap(24,24).toImage();
    }

    m_pVolumeBtn->refreshIcon(image, color);
}

QFrame* UkmediaAppItemWidget::myLine()
{
    QFrame *line = new QFrame(this);
    line->setFixedSize(512, 1);
    line->setLineWidth(0);
    line->setFrameShape(QFrame::HLine);
    line->setFrameShadow(QFrame::Sunken);
    return line;
}

UkmediaAppItemWidget::~UkmediaAppItemWidget()
{

}
