#ifndef COMMON_H
#define COMMON_H

#include <QDBusArgument>
#include <QLabel>

#define PA_VOLUME_NORMAL 65536.0
#define UKMEDIA_VOLUME_NORMAL 100.0
#define SYSTEM_VOLUME_CTRL "kylin-settings-system"
#define UKUI_MEDIA_PATH "/org/ukui/media"
#define UKUI_MEDIA_CONTROL_PATH "/org/ukui/media/control"
#define UKUI_MEDIA_SERVICE "org.ukui.media"
#define UKUI_MEDIA_INTERFACE "org.ukui.media"
#define PULSEAUDIO_DEVICECONTROL_PATH "/org/pulseaudio/device_control"
#define PULSEAUDIO_DEVICECONTROL_SERVICE "org.PulseAudio.DeviceControl"
#define PULSEAUDIO_DEVICECONTROL_INTERFACE "org.PulseAudio.DeviceControl"

//设备插入状态
typedef enum pa_device_plugged_stauts
{
    PORT_STAUTS_UNKNOWN   = 0,
    PORT_STAUTS_PLUGGED   = 1,
    PORT_STAUTS_UNPLUGGED = 2,

} pa_device_plugged_stauts;

//文本长自动省略并添加悬浮
class FixLabel : public QLabel
{
        Q_OBJECT
    public:
        explicit FixLabel(QWidget *parent = nullptr);
        explicit FixLabel(QString text , QWidget *parent = nullptr);
        ~FixLabel();
        void setText(const QString &text, bool saveTextFlag = true);
        QString fullText;
    private:
        void paintEvent(QPaintEvent *event);

    private:
};

enum SoundType {
    SINK,
    SOURCE,
};

struct pa_device_port_info
{
    /*card index*/
    int card;

    /* port direction. Input or Output */
    int direction;

    /* port available. */
    int available;

    /* port plugged stauts. */
    int plugged_stauts;

    /* port name. Example: analog-output-speak */
    QString name;

    /* port description */
    QString description;

    /* device description */
    QString device_description;

    /* device product name */
    QString device_product_name;

    friend const QDBusArgument& operator>>(const QDBusArgument &argument, pa_device_port_info &info) {
        argument.beginStructure();
        argument >> info.card;
        argument >> info.direction;
        argument >> info.available;
        argument >> info.plugged_stauts;
        argument >> info.name;
        argument >> info.description;
        argument >> info.device_description;
        argument >> info.device_product_name;
        argument.endStructure();
        return argument;
    }

    friend const QDBusArgument &operator<<(QDBusArgument &argument, const pa_device_port_info &info) {
        argument.beginStructure();
        argument << info.card;
        argument << info.direction;
        argument << info.available;
        argument << info.plugged_stauts;
        argument << info.name;
        argument << info.description;
        argument << info.device_description;
        argument << info.device_product_name;
        argument.endStructure();
        return argument;
    }
};

typedef pa_device_port_info pa_device_port_info;
Q_DECLARE_METATYPE(pa_device_port_info)

/* some information of sink-input or source-output */
struct appInfo
{
    int index;

    int volume;

    int channel;

    bool mute;

    int direction;

    /* name eg: kylin-music*/
    QString name;

    /* sink-input's sink index*/
    int masterIndex;

    /* sink-input's sink name*/
    QString masterDevice;

    friend QDBusArgument &operator<<(QDBusArgument &argument, const appInfo &mystruct)
    {
        argument.beginStructure();
        argument << mystruct.index << mystruct.volume << mystruct.channel << mystruct.mute
                 << mystruct.direction << mystruct.name << mystruct.masterIndex << mystruct.masterDevice;
        argument.endStructure();
        return argument;
    }

    friend const QDBusArgument &operator>>(const QDBusArgument &argument, appInfo &mystruct)
    {
        argument.beginStructure();
        argument >> mystruct.index >> mystruct.volume >> mystruct.channel >> mystruct.mute
                 >> mystruct.direction >> mystruct.name >> mystruct.masterIndex >> mystruct.masterDevice;
        argument.endStructure();
        return argument;
    }
};

typedef appInfo appInfo;
Q_DECLARE_METATYPE(appInfo)

#endif // COMMON_H
