/*
 * Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
 *
 */
#ifndef CUSTOMSOUND_H
#define CUSTOMSOUND_H
//#include <QtXml>
#include <QDomDocument>
#include <QDir>
#include <QFile>
#include <QDebug>
#include <QDBusArgument>

enum SoundType {
    SINK,
    SOURCE,
};

/* some information of sink-input or source-output */
struct appInfo
{
    int index;

    int volume;

    int channel;

    bool mute;

    int direction;

    /* name eg: kylin-music*/
    QString name;

    /* sink-input's sink index*/
    int masterIndex;

    /* sink-input's sink name*/
    QString masterDevice;

    friend QDBusArgument &operator<<(QDBusArgument &argument, const appInfo &mystruct)
    {
        argument.beginStructure();
        argument << mystruct.index << mystruct.volume << mystruct.channel << mystruct.mute
                 << mystruct.direction << mystruct.name << mystruct.masterIndex << mystruct.masterDevice;
        argument.endStructure();
        return argument;
    }

    friend const QDBusArgument &operator>>(const QDBusArgument &argument, appInfo &mystruct)
    {
        argument.beginStructure();
        argument >> mystruct.index >> mystruct.volume >> mystruct.channel >> mystruct.mute
                 >> mystruct.direction >> mystruct.name >> mystruct.masterIndex >> mystruct.masterDevice;
        argument.endStructure();
        return argument;
    }
};

typedef appInfo appInfo;
Q_DECLARE_METATYPE(appInfo)

class CustomSound
{
public:
    CustomSound();
    ~CustomSound();
    bool createAudioFile();
    bool isExist(QString nodeName);
    int addXmlNode(QString nodeNane, bool initState);
    bool isFirstRun();


public:
    QFile *file;
private:
    QDomDocument *doc;
    QString audioPath;
};

#endif // CUSTOMSOUND_H
